# -*- coding: utf-8 -*-

# Copyright (C) 2020 Leodanis Pozo Ramos <lpozor78@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

"""This module provides the OilBlend class."""

from typing import Dict

from lubepy import MIN_OIL_DENSITY, ASH_CONTRIBUTION
from lubepy.validator.core import OilDensity, AdditivePercent, MetalContent


def additive_percent_mass(
    additive_percent: float, additive_density: float, oil_density: float
) -> float:
    """Calculate the % by mass of Additive in a motor oil."""
    return OilBlend(
        additive_percent, additive_density, oil_density, {}
    ).additive_percent_mass()


def total_ash(metal_content: dict, additive_percent: float) -> float:
    """Calculate the total content of sulfated ash."""
    return OilBlend(
        additive_percent, MIN_OIL_DENSITY, MIN_OIL_DENSITY, metal_content
    ).total_ash()


class OilBlend:
    """Class to calculate some parameters of a motor oil blend."""

    additive_percent = AdditivePercent("Additive percent")
    additive_density = OilDensity("Additive density")
    oil_density = OilDensity("Oil density")
    metal_content = MetalContent("Metal content")

    def __init__(
        self,
        additive_percent: float,
        additive_density: float,
        oil_density: float,
        metal_content: Dict[str, float],
    ) -> None:
        """Class initializer.

        additive_percent: Total % of additive in the blend (% volume)
        additive_density: Additive package density (kg/L)
        oil_density: OilDensity of the finished oil (kg/L)
        metal_content: Metallic additive content (% mass)
            e.g {'Calcium': 0.47, 'Magnesium': 1.15, 'Zinc': 1.66}
        """
        self.additive_percent = additive_percent
        self.additive_density = additive_density
        self.oil_density = oil_density
        self.metal_content = metal_content

    def additive_percent_mass(self) -> float:
        """Calculate the % by mass of Additive in a motor oil.

                                Additive OilDensity (kg/L) * Additive (% volume)
        Additive (% mass) = ---------------------------------------------------
                                       OilDensity of Finished Oil (kg/L)
        """
        return round(
            (self.additive_density * self.additive_percent) / self.oil_density,
            2,
        )

    def _ash_per_metal(self, metal: str) -> float:
        """Calculate the % of Sulfated Ash (SA) of a motor oil.

             Metal Content (%mass) * Ash Contribution * Additive Pkg (%volume)
        SA = -----------------------------------------------------------------
                                            100

        metal: Any of these: 'zinc', 'barium', 'sodium', 'calcium',
               'magnesium', 'lead', 'boron', 'potassium',
               'manganese', 'molybdenum', 'copper'
        """
        return round(
            self.metal_content[metal]
            * ASH_CONTRIBUTION[metal.lower()]
            * self.additive_percent
            / 100,
            3,
        )

    def total_ash(self) -> float:
        """Calculate the total content of sulfated ash."""
        return round(
            sum(self._ash_per_metal(metal) for metal in self.metal_content), 2
        )



# -*- coding: utf-8 -*-

# Copyright (C) 2020 Leodanis Pozo Ramos <lpozor78@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

"""This module provides viscosity calculations."""

import math

from lubepy import MAX_VISCOSITY_40, MAX_VISCOSITY_100, MIN_VISCOSITY
from lubepy.validator.core import (
    validate_viscosity,
    validate_viscosity_index,
    validate_temperature,
)


def viscosity_at_40(viscosity100: float, index: float) -> float:
    """Calculate the Kinematic Viscosity (KV) at 40°C.

    Valid for viscosities under 2000 cSt at 40°C.
    """
    _viscosity = _viscosity100 = validate_viscosity(viscosity100, "100")
    _index = validate_viscosity_index(index)
    while (
        _viscosity_index(_viscosity, _viscosity100) >= _index
        and _viscosity <= MAX_VISCOSITY_40
    ):
        _viscosity += 0.05

    return round((_viscosity * 100 + 0.1) / 100, 2)


def viscosity_at_100(viscosity40: float, index: float) -> float:
    """Calculate the Kinematic Viscosity (KV) at 100°C.

    Valid for viscosities between 2 and 500 cSt at 100°C.
    """
    _viscosity = MIN_VISCOSITY
    _viscosity40 = validate_viscosity(viscosity40, "40")
    _index = validate_viscosity_index(index)
    while (
        _viscosity_index(_viscosity40, _viscosity) <= _index
        and _viscosity <= MAX_VISCOSITY_100
    ):
        _viscosity += 0.01

    return round((_viscosity * 100 + 0.01) / 100, 2)


_TO_KELVIN = 273.15


def viscosity_at_any_temp(
    viscosity40: float, viscosity100: float, temperature: float
) -> float:
    """Calculate the kinematic viscosity at any temperature (ASTM D341)."""
    _viscosity40 = validate_viscosity(viscosity40, "40")
    _viscosity100 = validate_viscosity(viscosity100, "100")
    _temperature = validate_temperature(temperature)

    x = math.log10(math.log10(_viscosity40 + 0.7))
    y = math.log10(math.log10(_viscosity100 + 0.7))
    t0 = math.log10(40 + _TO_KELVIN)
    t1 = math.log10(100 + _TO_KELVIN)
    target_t = math.log10(_temperature + _TO_KELVIN)
    b = (x - y) / (t1 - t0)
    a = x + b * t0
    v = 10 ** (10 ** (a - b * target_t)) - 0.7

    return round(v, 2)


def viscosity_index(viscosity40: float, viscosity100: float) -> float:
    """Calculate the Viscosity Index (VI) by ASTM-D2270.

    - Viscosity Index Up to and Including 100

           (L - KV40)
    VI = ------------- * 100
            (L - H)

    where:
        KV40:  kinematic viscosity at 40°C of the oil whose viscosity
                index is to be calculated mm^2/s (cSt).
        L: kinematic viscosity at 40°C of an oil of 0 viscosity
            index having the same kinematic viscosity at 100°C as
            the oil whose viscosity index is to be calculated,
            mm^2/s (cSt)
        H: kinematic viscosity at 40°C of an oil of 100 viscosity
            index having the same kinematic viscosity at 100°C as
            the oil whose viscosity index is to be calculated mm 2 /s
            (cSt)

    L = a * KV100^2 + b * KV100 + c

    where:
        KV100: kinematic viscosity at 100°C of the oil whose viscosity
                index is to be calculated, mm^2/s (cSt)
        a, b, c: interpolation coefficients

    H = d * KV100^2 + e * KV100 + f
    where:
        d, e, f: interpolation coefficients

    - Viscosity Index of 100 and Greater

            (10^N - 1)
    VI = --------------- + 100
             0.00715

    where:
              log10(H) - log10(KV40)
        N = --------------------------
                  log10(KV100)
    """
    return _viscosity_index(viscosity40, viscosity100)


def _viscosity_index(viscosity40: float, viscosity100: float) -> float:
    """Calculate the Viscosity Index (VI) by ASTM-D2270."""
    _viscosity40 = validate_viscosity(viscosity40, "40")
    _viscosity100 = validate_viscosity(viscosity100, "100")
    upper = float("inf")
    interpolation_coefs = {
        (2.0, 3.8): (1.14673, 1.7576, -0.109, 0.84155, 1.5521, -0.077),
        (3.8, 4.4): (3.38095, -15.4952, 33.196, 0.78571, 1.7929, -0.183),
        (4.4, 5.0): (2.5, -7.2143, 13.812, 0.82143, 1.5679, 0.119),
        (5.0, 6.4): (0.101, 16.635, -45.469, 0.04985, 9.1613, -18.557),
        (6.4, 7.0): (3.35714, -23.5643, 78.466, 0.22619, 7.7369, -16.656),
        (7.0, 7.7): (0.01191, 21.475, -72.870, 0.79762, -0.7321, 14.61),
        (7.7, 9.0): (0.41858, 16.1558, -56.040, 0.05794, 10.5156, -28.240),
        (9.0, 12.0): (0.88779, 7.5527, -16.600, 0.26665, 6.7015, -10.810),
        (12.0, 15.0): (0.7672, 10.7972, -38.180, 0.20073, 8.4658, -22.490),
        (15.0, 18.0): (0.97305, 5.3135, -2.200, 0.28889, 5.9741, -4.930),
        (18.0, 22.0): (0.97256, 5.25, -0.980, 0.24504, 7.416, -16.730),
        (22.0, 28.0): (0.91413, 7.4759, -21.820, 0.20323, 9.1267, -34.230),
        (28.0, 40.0): (0.87031, 9.7157, -50.770, 0.18411, 10.1015, -46.750),
        (40.0, 55.0): (0.84703, 12.6752, -133.310, 0.17029, 11.4866, -80.620),
        (55.0, 70.0): (0.85921, 11.1009, -83.19, 0.1713, 11.368, -76.940),
        (70.0, upper): (0.83531, 14.6731, -216.246, 0.16841, 11.8493, -96.947),
    }

    a, b, c, d, e, f = [0.0] * 6

    for k, v in interpolation_coefs.items():
        if k[0] <= _viscosity100 < k[1]:
            a, b, c, d, e, f = v
            break

    L = a * _viscosity100 ** 2 + b * _viscosity100 + c

    H = d * _viscosity100 ** 2 + e * _viscosity100 + f

    if _viscosity40 >= H:
        return round(((L - _viscosity40) / (L - H)) * 100)

    N = (math.log10(H) - math.log10(_viscosity40)) / math.log10(_viscosity100)

    return round(((10 ** N - 1) / 0.00715) + 100)



# -*- coding: utf-8 -*-

# Copyright (C) 2020 Leodanis Pozo Ramos <lpozor78@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

"""This module provides the OilMixture class."""

import math
from collections import namedtuple

from lubepy.exceptions import ConceptError
from lubepy.validator.core import Temperature, validate_viscosity


def mixture_viscosity(
    first_viscosity: float,
    first_oil_percent: float,
    second_viscosity: float,
    temperature: str,
) -> float:
    """Return the resulting viscosity of a mix of two base oils."""
    return OilMixture(
        first_viscosity, second_viscosity, temperature
    ).mixture_viscosity(first_oil_percent)


_Proportions = namedtuple(
    "_Proportions", ["first_oil_percent", "second_oil_percent"]
)


def mixture_proportions(
    first_viscosity: float,
    second_viscosity: float,
    desired_viscosity: float,
    temperature: str,
) -> _Proportions:
    """Return proportions to get a mixture of a given viscosity."""
    return OilMixture(
        first_viscosity, second_viscosity, temperature
    ).mixture_proportions(desired_viscosity)


class OilMixture:
    """Class to provide calculations on oil mixtures."""

    temperature = Temperature("Temperature")

    def __init__(
        self, first_viscosity: float, second_viscosity: float, temperature: str
    ) -> None:
        """Class initializer."""
        self.temperature = temperature
        self.first_viscosity = validate_viscosity(
            first_viscosity, self.temperature
        )
        self.second_viscosity = validate_viscosity(
            second_viscosity, self.temperature
        )
        self.temp_map = {"100": 1.8, "40": 4.1, "-5": 1.9}

    def mixture_viscosity(self, first_oil_percent: float) -> float:
        """Return the resulting viscosity of a mix of two base oils.

        Mixture KV = e ^ (a * e ^ (x1 * log(b / a))) - K

        Where:
            x1: Proportion of base oil # 1
            a = log(KV2 + K)
            b = log(KV1 + K)
                KV1, KV2: Kinematic Viscosity of oil # 1 and # 2 (cSt)
            K: Temperature correction

        """
        K = self.temp_map[self.temperature]
        x1 = first_oil_percent / 100
        a = math.log(self.second_viscosity + K)
        b = math.log(self.first_viscosity + K)
        mix_viscosity = math.exp(a * math.exp(x1 * math.log(b / a))) - K

        return round(mix_viscosity, 2)

    def mixture_proportions(self, desired_viscosity: float) -> _Proportions:
        """Return proportions to get a mixture of a given viscosity.

        first_oil_percent = 100 * (math.log(a / c) / math.log(b / c))

        Where:
            a = math.log(desired_viscosity + K)
            b = math.log(KV1 + K)
            c = math.log(KV2 + K)
                K: Temperature correction

        """

        desired_viscosity = validate_viscosity(
            desired_viscosity, self.temperature
        )

        if not (
            self.first_viscosity <= desired_viscosity <= self.second_viscosity
            or self.second_viscosity
            <= desired_viscosity
            <= self.first_viscosity
        ):
            raise ConceptError(
                "Mixture viscosity must be inside the viscosity interval"
            )

        K = self.temp_map[self.temperature]
        a = math.log(desired_viscosity + K)
        b = math.log(self.first_viscosity + K)
        c = math.log(self.second_viscosity + K)
        first_oil_percent = 100 * (math.log(a / c) / math.log(b / c))
        second_oil_percent = 100 - first_oil_percent

        return _Proportions(
            round(first_oil_percent, 2), round(second_oil_percent, 2)
        )



# -*- coding: utf-8 -*-

# Copyright (C) 2020 Leodanis Pozo Ramos <lpozor78@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

"""This module provides validators for input parameters."""

from math import isinf, isnan

from lubepy import (
    MIN_VISCOSITY,
    MAX_VISCOSITY_MINUS_5,
    MAX_VISCOSITY_40,
    MAX_VISCOSITY_100,
    MIN_VISCOSITY_INDEX,
    MAX_VISCOSITY_INDEX,
    MIN_TEMPERATURE,
    MAX_TEMPERATURE,
    MIN_BEARING_DIAMETER,
    MAX_BEARING_DIAMETER,
    MIN_BEARING_WIDTH,
    MAX_BEARING_WIDTH,
    MIN_RPM,
    MAX_RPM,
    MIN_OIL_DENSITY,
    MAX_OIL_DENSITY,
    MIN_ADDITIVE_PERCENT,
    MAX_ADDITIVE_PERCENT,
    MIN_FLOW_RATE,
    MAX_FLOW_RATE,
    MIN_PIPE_EQUIVALENT_DIAMETER,
    MAX_PIPE_EQUIVALENT_DIAMETER,
    ASH_CONTRIBUTION,
)

from lubepy.exceptions import ConceptError, ValidationError


def validate_viscosity(value, temperature: str) -> float:
    upper_limit = {
        "-5": MAX_VISCOSITY_MINUS_5,
        "40": MAX_VISCOSITY_40,
        "100": MAX_VISCOSITY_100,
    }
    validate = ParamValidator()
    return validate(
        param=f"Viscosity at {temperature}",
        value=value,
        lower=MIN_VISCOSITY,
        upper=upper_limit[temperature],
    )


def validate_viscosity_index(value) -> float:
    validate = ParamValidator()
    return validate(
        param="Viscosity Index",
        value=value,
        lower=MIN_VISCOSITY_INDEX,
        upper=MAX_VISCOSITY_INDEX,
    )


def validate_temperature(value) -> float:
    validate = ParamValidator()
    return validate(
        param="Temperature",
        value=value,
        lower=MIN_TEMPERATURE,
        upper=MAX_TEMPERATURE,
    )


class ParamValidator:
    """Validate params."""

    def __init__(self):
        self._param = None
        self._value = None

    def __call__(self, param, value, lower=None, upper=None):
        self._param = param
        self._value = value
        self._cleanup()
        self._to_float()
        self._check_symbols()
        self._check_range(lower, upper)
        return self._value

    def _cleanup(self):
        self._value = "".join(str(self._value).split()).replace(",", ".")

    def _to_float(self):
        try:
            self._value = float(self._value)
        except ValueError:
            raise ValidationError(self._error_msg) from None

    def _check_symbols(self):
        if isinf(self._value) or isnan(self._value):
            raise ValidationError(self._error_msg)

    def _check_range(self, lower: float = None, upper: float = None):
        if lower is None and upper is None:
            return None
        if not lower <= self._value <= upper:
            raise ConceptError(
                f"{self._param} must be between {lower} and {upper}"
            )

    @property
    def _error_msg(self):
        return f"{self._param} must be a valid number, not: {self._value}"


class BaseParam:
    def __init__(self, name):
        self._name = name
        self._value = None
        self._validate = ParamValidator()

    def __get__(self, instance, owner):
        return self._value


class Temperature(BaseParam):
    """Descriptor class for validating temperature."""

    def __set__(self, instance, value):
        temperature = str(value).strip()
        if temperature in {"-5", "40", "100"}:
            self._value = temperature
        else:
            raise ConceptError(f"{self._name} must be -5ºC, 40ºC or 100ºC")


class BearingDiameter(BaseParam):
    """Descriptor class for validating bearing diameters."""

    def __set__(self, instance, value):
        self._value = self._validate(
            self._name, value, MIN_BEARING_DIAMETER, MAX_BEARING_DIAMETER
        )


class BearingWidth(BaseParam):
    """Descriptor class for validating bearing widths."""

    def __set__(self, instance, value):
        self._value = self._validate(
            self._name, value, MIN_BEARING_WIDTH, MAX_BEARING_WIDTH
        )


class Rpm(BaseParam):
    """Descriptor class for validating bearing rpm."""

    def __set__(self, instance, value):
        self._value = self._validate(self._name, value, MIN_RPM, MAX_RPM)


class OilDensity(BaseParam):
    """Descriptor class for validating density in g/ml."""

    def __set__(self, instance, value):
        self._value = self._validate(
            self._name, value, MIN_OIL_DENSITY, MAX_OIL_DENSITY
        )


class AdditivePercent(BaseParam):
    """Descriptor class for validating additive percent."""

    def __set__(self, instance, value):
        self._value = self._validate(
            self._name, value, MIN_ADDITIVE_PERCENT, MAX_ADDITIVE_PERCENT
        )


class MetalContent(BaseParam):
    """Descriptor class for validating metal content dict."""

    def __set__(self, instance, value):
        if not isinstance(value, dict):
            raise TypeError(
                f"{self._name} must be a dictionary object not {type(value)}"
            )

        self._value = {
            k.strip().lower(): self._validate(f"{self._name} for {k}", v)
            for k, v in value.items()
        }

        for metal in self._value:
            if metal not in ASH_CONTRIBUTION:
                raise ConceptError(f"{metal} is not a valid additive metal")


class FlowRate(BaseParam):
    """Descriptor class for validating a flow velocity in m/s."""

    def __set__(self, instance, value):
        self._value = self._validate(
            self._name, value, MIN_FLOW_RATE, MAX_FLOW_RATE
        )


class PipeSession(BaseParam):
    """Descriptor class for validating the session of a pipe in mm."""

    def __set__(self, instance, value):
        self._value = self._validate(
            self._name,
            value,
            MIN_PIPE_EQUIVALENT_DIAMETER,
            MAX_PIPE_EQUIVALENT_DIAMETER,
        )



# -*- coding: utf-8 -*-

# Copyright (C) 2020 Leodanis Pozo Ramos <lpozor78@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

"""This module provides the Reynold number calculations."""

from enum import Enum

from lubepy.validator.core import (
    FlowRate,
    PipeSession,
)
from lubepy import MIN_PIPE_EQUIVALENT_DIAMETER
from lubepy.lube.viscosity import viscosity_at_any_temp
from lubepy.validator.core import ParamValidator


def reynolds_circular_session(
    flow_rate: float,
    viscosity40: float,
    viscosity100: float,
    temperature: float,
    diameter: float,
) -> float:
    """Calculate Reynolds number (Re)."""
    return ReynoldsNumber(
        flow_rate, viscosity40, viscosity100, temperature
    ).reynolds_circular_session(diameter)


def reynolds_square_session(
    velocity: float,
    viscosity40: float,
    viscosity100: float,
    temperature: float,
    side: float,
) -> float:
    """Calculate Reynolds number (Re)."""
    return ReynoldsNumber(
        velocity, viscosity40, viscosity100, temperature
    ).reynolds_square_session(side)


def reynolds_rectangular_session(
    velocity: float,
    viscosity40: float,
    viscosity100: float,
    temperature: float,
    base: float,
    height: float,
) -> float:
    """Calculate Reynolds number (Re)."""
    return ReynoldsNumber(
        velocity, viscosity40, viscosity100, temperature
    ).reynolds_rectangular_session(base, height)


def flow_type_circular_session(
    velocity: float,
    viscosity40: float,
    viscosity100: float,
    temperature: float,
    diameter: float,
) -> str:
    """Determine the flow type of a fluid."""
    return FluidFlowType(
        velocity, viscosity40, viscosity100, temperature
    ).flow_type_circular_session(diameter)


def flow_type_square_session(
    velocity: float,
    viscosity40: float,
    viscosity100: float,
    temperature: float,
    side: float,
) -> str:
    """Determine the flow type of a fluid."""
    return FluidFlowType(
        velocity, viscosity40, viscosity100, temperature
    ).flow_type_square_session(side)


def flow_type_rectangular_session(
    velocity: float,
    viscosity40: float,
    viscosity100: float,
    temperature: float,
    base: float,
    height: float,
) -> str:
    """Determine the flow type of a fluid."""
    return FluidFlowType(
        velocity, viscosity40, viscosity100, temperature
    ).flow_type_rectangular_session(base, height)


class ReynoldsNumber:
    """Class for calculations related to Reynolds number."""

    _flow_rate = FlowRate("Flow rate")
    _equivalent_diameter = PipeSession("Pipe equivalent diameter")

    def __init__(
        self,
        flow_rate: float,
        viscosity40: float,
        viscosity100: float,
        temperature: float,
    ) -> None:
        self._flow_rate = flow_rate
        self._viscosity = viscosity_at_any_temp(
            viscosity40, viscosity100, temperature
        )
        self._equivalent_diameter = MIN_PIPE_EQUIVALENT_DIAMETER

    def _reynolds_number(self, equivalent_diameter: float) -> float:
        """Calculate Reynolds number (Re).

              K * Q
        Re = --------  => non-unit
              Lc * v
        where:
            K: Conversion factor = 353.9606
            Q: Flow rate (L/h)
            Lc: characteristic length (mm)
                For circular session Lc = D
                    D: diameter
                For square session Lc = L
                    L: side
                                             (2 * a * b)
                For rectangular session Lc = -----------
                                               (a + b)
                    a, b: sides
            v: Kinematic Viscosity (cSt)
        """
        K = 353.9606
        self._equivalent_diameter = equivalent_diameter

        return round(
            (K * self._flow_rate)
            / (self._equivalent_diameter * self._viscosity),
            1,
        )

    def _reynolds_circular_square(self, equivalent_diameter):
        return self._reynolds_number(equivalent_diameter=equivalent_diameter)

    def reynolds_circular_session(self, diameter):
        return self._reynolds_circular_square(equivalent_diameter=diameter)

    def reynolds_square_session(self, side):
        return self._reynolds_circular_square(equivalent_diameter=side)

    def reynolds_rectangular_session(self, base, height):
        validate = ParamValidator()
        base = validate("Base", base)
        height = validate("Height", height)
        return self._reynolds_number(
            equivalent_diameter=(2 * base * height) / (base + height)
        )


class _FlowTypes(Enum):
    """Enumeration to represent the flow type."""

    LAMINAR: str = "laminar"
    TURBULENT: str = "turbulent"
    MIXED: str = "mixed"


class FluidFlowType:
    def __init__(
        self,
        flow_rate: float,
        viscosity40: float,
        viscosity100: float,
        temperature: float,
    ) -> None:
        self._reynolds_number = ReynoldsNumber(
            flow_rate, viscosity40, viscosity100, temperature,
        )

    @staticmethod
    def _flow_type(number: float) -> str:
        """Determine the flow type of a fluid.

        Re <= 2100 => Laminar flow
        2100.0 < reynolds <= 4000.0 => Mixed flow
        Re > 4000 => Turbulent flow
        """
        if number <= 2_100.0:
            return _FlowTypes.LAMINAR
        if number > 4_000.0:
            return _FlowTypes.TURBULENT

        return _FlowTypes.MIXED

    def flow_type_circular_session(self, diameter):
        number = self._reynolds_number.reynolds_circular_session(diameter)
        return self._flow_type(number)

    def flow_type_square_session(self, side):
        number = self._reynolds_number.reynolds_square_session(side)
        return self._flow_type(number)

    def flow_type_rectangular_session(self, base, height):
        number = self._reynolds_number.reynolds_rectangular_session(
            base, height
        )
        return self._flow_type(number)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2020 Leodanis Pozo Ramos <lpozor78@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

"""This module provides the setup.py script."""

from pathlib import Path
from typing import Dict

from setuptools import find_packages
from setuptools import setup

__here__ = Path().cwd()

# Load the package's __init__.py module as a dictionary.
__about__: Dict = {}
with Path(__here__, "lubepy", "__init__.py").open(encoding="UTF-8") as f:
    exec(f.read(), __about__)

# Import the README.md and use it as the long-description.
try:
    with Path(__here__, "README.md").open(encoding="utf-8") as f:
        __long_description__ = "\n" + f.read()
except FileNotFoundError:
    __long_description__ = __about__["DESCRIPTION"]


setup(
    name=__about__["NAME"],
    version=__about__["VERSION"],
    description=__about__["DESCRIPTION"],
    long_description=__long_description__,
    long_description_content_type="text/markdown",
    author=__about__["AUTHOR"],
    author_email=__about__["EMAIL"],
    python_requires=__about__["REQUIRES_PYTHON"],
    url=__about__["URL"],
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    license="GNU General Public License, Version 2, June 1991",
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
    keywords="machinery lubrication, oil, grease, lubrication engineering",
    platforms=["linux", "darwin"],
)



# -*- coding: utf-8 -*-

# Copyright (C) 2020 Leodanis Pozo Ramos <lpozor78@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

"""This module provides the lubepy package."""

# Package meta-data.
NAME = "lubepy"
DESCRIPTION = (
    "Lubepy provides a basic set of machinery lubrication calculations."
)
URL = "https://github.com/lpozo/lubepy"
EMAIL = "lpozor78@gmail.com"
AUTHOR = "Leodanis Pozo Ramos"
REQUIRES_PYTHON = ">=3.6"
VERSION = "1.0a0"

# Package constants
MIN_VISCOSITY = 2.0  # cSt
MAX_VISCOSITY_MINUS_5 = 10_000.00  # cSt
MAX_VISCOSITY_40 = 2_000.0  # cSt
MAX_VISCOSITY_100 = 500.0  # cSt
MIN_VISCOSITY_INDEX = -25.0  # No unit
MAX_VISCOSITY_INDEX = 400.0  # No unit
MIN_TEMPERATURE = -50.0  # ºC
MAX_TEMPERATURE = 1_200.0  # ºC
MIN_BEARING_DIAMETER = 1.0  # mm
MAX_BEARING_DIAMETER = 100_000.0  # mm
MIN_BEARING_WIDTH = 1.0  # mm
MAX_BEARING_WIDTH = 10_000.0  # mm
MIN_PIPE_EQUIVALENT_DIAMETER = 1.0  # mm
MAX_PIPE_EQUIVALENT_DIAMETER = 1_000.0  # mm
MIN_RPM = 1.0  # 1/s
MAX_RPM = 300_000.0  # 1/s
MIN_ADDITIVE_PERCENT = 0.0  # %
MAX_ADDITIVE_PERCENT = 50.0  # %
MIN_OIL_DENSITY = 0.7  # g/mL
MAX_OIL_DENSITY = 1.5  # g/mL
MIN_FLOW_RATE = 0.1  # L/h
MAX_FLOW_RATE = 5_000.0  # L/h
ASH_CONTRIBUTION = {
    "zinc": 1.50,
    "barium": 1.70,
    "sodium": 3.09,
    "calcium": 3.40,
    "magnesium": 4.95,
    "lead": 1.464,
    "boron": 3.22,
    "potassium": 2.23,
    "manganese": 1.291,
    "molybdenum": 1.5,
    "copper": 1.252,
}



# -*- coding: utf-8 -*-

# Copyright (C) 2020 Leodanis Pozo Ramos <lpozor78@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

"""This module provides the Bearing class."""

import math
from typing import Dict, Tuple

from lubepy import (
    MAX_BEARING_DIAMETER,
    MIN_BEARING_DIAMETER,
    MIN_BEARING_WIDTH,
    MIN_RPM,
)
from lubepy.exceptions import ConceptError
from lubepy.validator.core import BearingDiameter, Rpm, BearingWidth


def grace_amount(outer_diameter: float, width: float) -> float:
    """Return the amount of grease (g) needed for re-lubrication."""
    return Bearing(outer_diameter, MIN_BEARING_DIAMETER, width).grease_amount()


def lubrication_frequency(
    inner_diameter: float, rpm: float, factors: Dict[str, int]
) -> float:
    """Return the amount of grease (g) needed for re-lubrication."""
    bearing = Bearing(MAX_BEARING_DIAMETER, inner_diameter, MIN_BEARING_WIDTH)
    return bearing.lubrication_frequency(rpm, factors)


def velocity_factor(
    outer_diameter: float, inner_diameter: float, rpm: float
) -> float:
    """Calculate the velocity factor of a bearing."""
    bearing = Bearing(outer_diameter, inner_diameter, MIN_BEARING_WIDTH)
    return bearing.velocity_factor(rpm)


class Bearing:
    """Class to define calculations related with bearings."""

    outer_diameter = BearingDiameter("Bearing outer diameter")
    inner_diameter = BearingDiameter("Bearing inner diameter")
    width = BearingWidth("Bearing BearingWidth")
    rpm = Rpm("Bearing rpm")

    def __init__(
        self, outer_diameter: float, inner_diameter: float, width: float
    ) -> None:
        """Class initializer."""
        self.outer_diameter = outer_diameter
        self.inner_diameter = inner_diameter
        if self.outer_diameter <= self.inner_diameter:
            raise ConceptError(
                "Outer diameter must be greater than inner diameter"
            )
        self.width = width
        self.rpm = MIN_RPM

    def grease_amount(self) -> float:
        """Return the amount of grease (g) needed for re-lubrication.

        Gg = 0.005 * D * B
        where:
            Gg: Amount of grease needed for re-lubrication (g)
            D: Outer diameter of bearing (mm)
            B: Total width of bearing (mm)
        """
        unit_coefficient = 0.005

        return round(unit_coefficient * self.outer_diameter * self.width, 2)

    def lubrication_frequency(
        self, rpm: float, factors: Dict[str, int]
    ) -> float:
        """Calculate the re-lubrication frequency in hours.

                       14000000
        T = K * [(------------------) - 4 * d]
                      n * sqrt(d)

        where:
        T: Frequency of re-lubrication (hours)
        K: Corrections factors
            K = Ft * Fc * Fh * Fv * Fp * Fd
            where:
            Ft: Temperature of bearing housing factor
                < 65°C, then Ft = 1.0
                65 to 80°C, then Ft = 0.5
                80 to 93°C, then Ft = 0.2
                > 93°C, then Ft = 0.1
            Fc: Solid contamination factor
                Light, no abrasive dust, then Fc = 1.0
                Severe, no abrasive dust, then Fc = 0.7
                Light, abrasive dust, then Fc = 0.4
                Severe, abrasive dust, then Fc = 0.2
            Fh: Moisture factor
                < 80 % ,then Fh = 1.0
                80 to 90 %, then Fh = 0.7
                Occasional condensation, then Fh = 0.4
                Water, then Fh = 0.2
            Fv: Vibrations factor
                Top velocity < 0.5 cm/s, then Fv = 1.0
                0.5 to 1.0 cm/s, then Fv = 0.6
                > 1.0 cm/s, then Fv = 0.3
            Fp: Shaft position factor
                Horizontal, then Fp = 1.0
                45 degrees, then Fp = 0.5
                Vertical, then Fp = 0.3
            Fd: Bearing design factor
                Ball bearing, then Fd = 10
                Cylinder/Needle roller bearing, then Fd = 5
                Conical roller bearing, then Fd = 1
        n: Rotation velocity (rpm)
        d: Inner diameter of the bearing (mm)
        """
        self.rpm = rpm

        correlation_factors: Dict[str, Tuple[float, ...]] = {
            "ft": (1.0, 0.5, 0.2, 0.1),
            "fc": (1.0, 0.7, 0.4, 0.2),
            "fh": (1.0, 0.7, 0.4, 0.1),
            "fv": (1.0, 0.6, 0.3),
            "fp": (1.0, 0.5, 0.3),
            "fd": (10.0, 5.0, 1.0),
        }

        k_factor = 1.0

        for factor, score_index in factors.items():
            k_factor *= correlation_factors[factor][score_index]

        frequency = k_factor * (
            (14000000 / (self.rpm * math.sqrt(self.inner_diameter)))
            - 4 * self.inner_diameter
        )

        return round(frequency)

    def velocity_factor(self, rpm: float) -> float:
        """Calculate the velocity factor of a bearing.

        A = n * dm

        where:
            A: Velocity factor (mm/min)
            n: Rotation velocity (rpm)
            dm: Mean diameter (mm)
                   (D + d)
            dm = -----------
                      2
            where:
                D: outer diameter
                d: inner diameter
        """
        self.rpm = rpm

        return round(
            self.rpm * (self.outer_diameter + self.inner_diameter) / 2
        )
